@@ -18,13 +18,13 @@
  *   along with Bless; if not, write to the Free Software
  *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-#define USING_GTK
 
 using System.Collections;
 using System;
 using System.IO;
 using System.Threading;
 using Bless.Util;
+using Bless.Tools;
 
 namespace Bless.Buffers {
 
@@ -33,7 +33,7 @@ namespace Bless.Buffers {
 /// It supports undo-redo and can easily handle large files.
 /// Editing is also very cheap.
 ///</summary>
-public class ByteBuffer : IBuffer {
+public class ByteBuffer : BaseBuffer {
 
 	///<summary>The file buffer associated with the ByteBuffer</summary>
 	internal FileBuffer	fileBuf;
@@ -87,48 +87,29 @@ public class ByteBuffer : IBuffer {
 	///<summary>Emitted when buffer permissions change</summary>
 	public event ChangedHandler PermissionsChanged;
 	
-	// Wrappers to emit events
-	// if gtk/glib is available make sure events are called in the main GUI thread
 	public void EmitChanged()
 	{
-		if (emitEvents && Changed!=null)
-#if USING_GTK
-			Gtk.Application.Invoke(delegate {
-				Changed(this);
-			});
-#else
+		if (emitEvents && Changed != null) {
 			Changed(this);
-#endif
+		}
 	}
 
 	public void EmitFileChanged()
 	{
-		if (emitEvents && FileChanged!=null)
-#if USING_GTK
-			Gtk.Application.Invoke(delegate {
-				FileChanged(this);
-			});
-#else
+		if (emitEvents && FileChanged != null) {
 			FileChanged(this);
-#endif
+		}
 	}
 	
 	public void EmitPermissionsChanged()
 	{
-		if (emitEvents && PermissionsChanged!=null)
-#if USING_GTK
-			Gtk.Application.Invoke(delegate {
-				PermissionsChanged(this);
-			});
-#else
+		if (emitEvents && PermissionsChanged != null) {
 			PermissionsChanged(this);
-#endif
+		}
 	}
 	
 	// related to asynchronous save model
 	AsyncCallback userSaveAsyncCallback;
-	AsyncCallback userSaveAsAsyncCallback;
-	AutoResetEvent saveAsFinishedEvent;
 	AutoResetEvent saveFinishedEvent;
 	bool useGLibIdle;
 	
@@ -150,7 +131,6 @@ public class ByteBuffer : IBuffer {
 		modifyAllowed = true;
 		
 		saveFinishedEvent = new AutoResetEvent(false);
-		saveAsFinishedEvent = new AutoResetEvent(false);
 		useGLibIdle = false;
 		emitEvents = true;
 		
@@ -237,10 +217,11 @@ public class ByteBuffer : IBuffer {
 	}
 	
 	///<summary>Append bytes at the end of the buffer</summary>
-	public void Append(byte[] data, long index, long length) 
+	public override void Append(byte[] data, long index, long length) 
 	{	
 		lock (LockObj) {
 			if (!modifyAllowed) return;
+			if (!IsResizable) return;
 		
 			AppendAction aa = new AppendAction(data, index, length, this);	
 			aa.Do();
@@ -256,19 +237,15 @@ public class ByteBuffer : IBuffer {
 		}
 	}
 
-	public void Append(byte[] data) 
-	{
-		Append(data, 0, data.Length);
-	}
-	
 	///<summary>Insert bytes into the buffer</summary>
-	public void Insert(long pos, byte[] data, long index, long length) 
+	public override void Insert(long pos, byte[] data, long index, long length) 
 	{
 		lock (LockObj) {
 			if (!modifyAllowed) return;
+			if (!IsResizable) return;
 			
 			if (pos == size) {
-				Append(data);
+				Append(data, index, length);
 				return;
 			}
 			
@@ -286,16 +263,12 @@ public class ByteBuffer : IBuffer {
 		}
 	}		
 
-	public void Insert(long pos, byte[] data)
-	{
-		Insert(pos, data, 0, data.Length);
-	}
-	
 	///<summary>Delete bytes from the buffer</summary>
 	public void Delete(long pos1, long pos2) 
 	{
 		lock (LockObj) {
 			if (!modifyAllowed) return;
+			if (!IsResizable) return;
 		
 			DeleteAction da = new DeleteAction(pos1, pos2, this);
 			da.Do();
@@ -317,6 +290,11 @@ public class ByteBuffer : IBuffer {
 		lock (LockObj) {
 			if (!modifyAllowed) return;
 			
+			// if the file isn't resizable allow replaces 
+			// only if they don't change the file length
+			bool equalLength = (pos2 - pos1 + 1 == length);
+			if (!IsResizable && !equalLength) return;
+			
 			ReplaceAction ra = new ReplaceAction(pos1, pos2, data, index, length, this);
 			ra.Do();
 			
@@ -380,8 +358,8 @@ public class ByteBuffer : IBuffer {
 		lock (LockObj) {
 			if (!fileOperationsAllowed) return null;
 			
-			saveAsFinishedEvent.Reset();
-			userSaveAsAsyncCallback = ac;
+			saveFinishedEvent.Reset();
+			userSaveAsyncCallback = ac;
 			
 			SaveAsOperation so = new SaveAsOperation(this, filename, progressCallback, SaveAsAsyncCallback, useGLibIdle);
 			
@@ -400,7 +378,7 @@ public class ByteBuffer : IBuffer {
 			saveThread.IsBackground = true;
 			saveThread.Start();
 			
-			return new ThreadedAsyncResult(so, saveAsFinishedEvent, false);
+			return new ThreadedAsyncResult(so, saveFinishedEvent, false);
 		}
 		
 	}
@@ -459,11 +437,11 @@ public class ByteBuffer : IBuffer {
 			EmitChanged();
 				
 			// if user provided a callback, call it now
-			if (userSaveAsAsyncCallback != null)
-				userSaveAsAsyncCallback(ar);
+			if (userSaveAsyncCallback != null)
+				userSaveAsyncCallback(ar);
 			
 			// notify that Save As has finished
-			saveAsFinishedEvent.Set();
+			saveFinishedEvent.Set();
 		}
 	}
 	
@@ -478,7 +456,20 @@ public class ByteBuffer : IBuffer {
 			saveFinishedEvent.Reset();
 			userSaveAsyncCallback = ac;
 			
-			SaveOperation so = new SaveOperation(this, TempFile.CreateName(tempDir), progressCallback, SaveAsyncCallback, useGLibIdle);
+			Thread saveThread = null;
+			ThreadedAsyncResult tar = null;
+
+			// decide whether to save in place or normally
+			if (!fileBuf.IsResizable || this.Size == fileBuf.Size) {
+				SaveInPlaceOperation sipo = new SaveInPlaceOperation(this, progressCallback, SaveInPlaceAsyncCallback, useGLibIdle);
+				saveThread = new Thread(sipo.OperationThread);
+				tar = new ThreadedAsyncResult(sipo, saveFinishedEvent, false);
+			}
+			else {
+				SaveOperation so = new SaveOperation(this, TempFile.CreateName(tempDir), progressCallback, SaveAsyncCallback, useGLibIdle);
+				saveThread = new Thread(so.OperationThread);
+				tar = new ThreadedAsyncResult(so, saveFinishedEvent, false);
+			}
 			
 			// don't allow messing up with the buffer
 			// while we are saving
@@ -490,12 +481,11 @@ public class ByteBuffer : IBuffer {
 			this.EmitEvents = false;
 			fsw.EnableRaisingEvents = false;
 			
-			// start save thread
-			Thread saveThread = new Thread(so.OperationThread);
+			// start save thread			
 			saveThread.IsBackground = true;
 			saveThread.Start();
 			
-			return new ThreadedAsyncResult(so, saveFinishedEvent, false);
+			return tar;
 		}
 	}
 	
@@ -507,8 +497,10 @@ public class ByteBuffer : IBuffer {
 		lock (LockObj) {
 			SaveOperation so = (SaveOperation)ar.AsyncState;
 			
-			
 			if (so.Result == SaveOperation.OperationResult.Finished) { // save went ok
+				// No need to call CloseFile() MakePrivateCopyOfUndoRedo()
+				// because it has already been called in SaveOperation
+
 				LoadWithFile(so.SavePath);
 				
 				if (undoDeque.Count > 0)
@@ -576,6 +568,52 @@ public class ByteBuffer : IBuffer {
 		}
 	}
 	
+	///<summary>
+	/// Called when an asynchronous in-place save operation finishes
+	///</summary>
+	void SaveInPlaceAsyncCallback(IAsyncResult ar)
+	{
+		lock (LockObj) {
+			SaveInPlaceOperation sipo = (SaveInPlaceOperation)ar.AsyncState;
+			
+			if (sipo.Result == ThreadedAsyncOperation.OperationResult.Finished) { // save went ok
+				LoadWithFile(sipo.SavePath);
+				
+				if (undoDeque.Count > 0)
+					SaveCheckpoint = undoDeque.PeekFront();
+				else
+					SaveCheckpoint = null;
+				
+				changedBeyondUndo = false;
+			}
+			else if (sipo.Result == ThreadedAsyncOperation.OperationResult.Cancelled) { // save cancelled
+				
+			}
+			else if (sipo.Result == ThreadedAsyncOperation.OperationResult.CaughtException) {
+				
+			}
+			
+			// re-allow buffer usage
+			this.ReadAllowed = true;
+			this.ModifyAllowed = true;
+			this.FileOperationsAllowed = true;
+			
+			this.EmitEvents = true;
+			fsw.EnableRaisingEvents = true;
+			
+			// notify the world about the changes			
+			EmitPermissionsChanged();
+			EmitChanged();			
+			
+			// if user provided a callback, call it now
+			if (userSaveAsyncCallback != null)
+				userSaveAsyncCallback(ar);
+			
+			// notify that Save has finished	
+			saveFinishedEvent.Set();
+		}
+	}
+	
 	///<summary> 
 	/// Revert ByteBuffer to the last saved state
 	///</summary> 
@@ -611,7 +649,7 @@ public class ByteBuffer : IBuffer {
 	/// Returns in a byte array the data contained in 
 	/// the specified range in the buffer.  
 	///</summary>
-	public byte[] RangeToByteArray(Range range)
+	public byte[] RangeToByteArray(IRange range)
 	{
 		if (range.Size==0)
 			return null;
@@ -667,11 +705,64 @@ public class ByteBuffer : IBuffer {
 		// can become invalid (eg after saving a file)
 		// Copy the data to private in-memory buffers to avoid data corruption
 		// and crashes...
-		foreach(ByteBufferAction action in undoDeque)
+		//
+
+		if (Preferences.Instance["Undo.KeepAfterSave"] == "Never") {
+			undoDeque.Clear();
+			redoDeque.Clear();
+			return;
+		}
+
+		if (Preferences.Instance["Undo.KeepAfterSave"] == "Always") {
+			foreach(ByteBufferAction action in undoDeque)
+				action.MakePrivateCopyOfData();
+
+			foreach(ByteBufferAction action in redoDeque)
+				action.MakePrivateCopyOfData();
+
+			return;
+		}
+
+		// if Preferences.Instance["Undo.KeepAfterSave"] == "Memory"
+		// drop the undo and redo actions that don't fit into memory (and all actions
+		// after them in the Deques).
+		Deque<ByteBufferAction> newUndoDeque = new Deque<ByteBufferAction>();
+		Deque<ByteBufferAction> newRedoDeque = new Deque<ByteBufferAction>();
+
+		foreach(ByteBufferAction action in undoDeque) {
+			long freeMem = long.MaxValue;
+			try {	
+				freeMem = Portable.GetAvailableMemory();
+			}
+			catch(NotImplementedException) {}
+
+			if (freeMem < action.GetPrivateCopySize())
+				break;
+
 			action.MakePrivateCopyOfData();
+			newUndoDeque.AddEnd(action);
+		}
 		
-		foreach(ByteBufferAction action in redoDeque)
+		
+		foreach(ByteBufferAction action in redoDeque) {
+			long freeMem = long.MaxValue;
+			try {	
+				freeMem = Portable.GetAvailableMemory();
+			}
+			catch(NotImplementedException) {}
+
+			if (freeMem < action.GetPrivateCopySize())
+				break;
+
 			action.MakePrivateCopyOfData();
+			newRedoDeque.AddEnd(action);
+		}
+
+		undoDeque.Clear();
+		redoDeque.Clear();
+
+		undoDeque = newUndoDeque;
+		redoDeque = newRedoDeque;
 	}
 	
 	private void SetupFSW()
@@ -697,17 +788,9 @@ public class ByteBuffer : IBuffer {
 		EmitFileChanged();
 	}
 	
-	public int Read(byte[] data, long pos, int len)
-	{
-		throw new NotImplementedException();
-	}
 	
-	public void Append(byte b)
-	{
-		throw new NotImplementedException();
-	}
 	
-	public byte this[long index] {
+	public override byte this[long index] {
 		set { } 
 		get {
 			lock (LockObj) {
@@ -750,7 +833,7 @@ public class ByteBuffer : IBuffer {
 		}
 	}
 	
-	public long Size {
+	public override long Size {
 		get { return size;}
 	}
 
@@ -831,6 +914,15 @@ public class ByteBuffer : IBuffer {
 		}
 	}
 	
+	// Whether buffer can be resized (read-only property)
+	public bool IsResizable {
+		get {
+			if (fileBuf != null)
+				return fileBuf.IsResizable;
+			else
+				return true;
+		}
+	}
 	// Use the GLib Idle handler for progress reporting.
 	// Mandatory if progress reporting involves Gtk+ widgets.
 	public bool UseGLibIdle {
