@@ -70,7 +70,7 @@ public class DataViewControl
 	{
 		Area clickArea = null;
 
-		foreach(Area a in dvDisplay.Layout.Areas) {
+		foreach(Area a in dvDisplay.Layout.AreaGroup.Areas) {
 			if ((x >= a.X) && (x <= a.X + a.Width)) {
 				clickArea = a;
 				break;
@@ -85,9 +85,9 @@ public class DataViewControl
 		//System.Console.WriteLine("* (1) Start: {0},{1},{2} End: {3},{4},{5}", selStartPos.First, selStartPos.Second, selStartPos.Digit, selEndPos.First, selEndPos.Second, selEndPos.Digit);
 		Bless.Util.Range r;
 		if (selStartPos.Second <= selEndPos.First)
-			r = new Bless.Util.Range(selStartPos.Second, selEndPos.First);
+			r = new Bless.Util.Range(ValidateOffset(selStartPos.Second), ValidateOffset(selEndPos.First));
 		else
-			r = new Bless.Util.Range(selEndPos.Second, selStartPos.First);
+			r = new Bless.Util.Range(ValidateOffset(selEndPos.Second), ValidateOffset(selStartPos.First));
 
 		//System.Console.WriteLine("Selection is ({0}, {1}) Expected ({2}, {3})", dataView.Selection.Start, dataView.Selection.End, r.Start, r.End);
 		// if nothing is selected and cursor position has changed externally
@@ -112,14 +112,7 @@ public class DataViewControl
 	private bool UpdateFocus(Area area)
 	{
 		if (area.CanFocus && !area.HasCursorFocus) {
-
-			foreach(Area a in dvDisplay.Layout.Areas)
-			a.HasCursorFocus = false;
-
-			dataView.MoveCursor(dataView.CursorOffset, dataView.CursorDigit);
-
-			area.HasCursorFocus = true;
-			area.MoveCursor(area.CursorOffset, area.CursorDigit);
+			dvDisplay.Layout.AreaGroup.FocusedArea = area;
 			return true;
 		}
 
@@ -165,6 +158,7 @@ public class DataViewControl
 	///</summary>
 	private void EvaluateSelection(DataViewDisplay.ShowType showType)
 	{
+		//System.Console.WriteLine("Evaluate (1) Start: {0},{1},{2} End: {3},{4},{5}", selStartPos.First, selStartPos.Second, selStartPos.Digit, selEndPos.First, selEndPos.Second, selEndPos.Digit);
 		long cursorOffset;
 
 		// no selection
@@ -179,6 +173,9 @@ public class DataViewControl
 		// selection with start pos <= end pos
 		else if (selStartPos.Second <= selEndPos.First) {
 			// set end position between bytes
+			// this is done so that selections performed with 
+			// the mouse can be continued with the keyboard
+			// (keyboard selections always set positions between bytes)
 			selEndPos.Second = selEndPos.First + 1;
 
 			// if selEndPos.Second is at or beyond the EOF
@@ -188,20 +185,42 @@ public class DataViewControl
 			cursorOffset = off;
 			dvDisplay.MakeOffsetVisible(cursorOffset, showType);
 
-			dataView.SetSelection(ValidateOffset(selStartPos.Second), ValidateOffset(selEndPos.First));
+			// set the selection only if the start position (which is the start of the selection)
+			// is within the file's limits
+			if (selStartPos.Second < dataView.Buffer.Size)
+				dataView.SetSelection(ValidateOffset(selStartPos.Second), ValidateOffset(selEndPos.First));
+			else
+				dataView.SetSelection(-1, -1);
+
 			dataView.MoveCursor(off, 0);
 		}
 		// selection with start pos > end pos
 		else {
-			long off = ValidateOffset(selEndPos.Second);
+			// set start position between bytes
+			// this is done so that selections performed with 
+			// the mouse can be continued with the keyboard
+			// (keyboard selections always set positions between bytes)
+			// TODO: This is currently disabled because it breaks mouse
+			// selection
+			//
+			//selStartPos.Second = selStartPos.First + 1;
+
+			long off = selEndPos.Second;
 			cursorOffset = off;
 			dvDisplay.MakeOffsetVisible(cursorOffset, showType);
 
-			dataView.SetSelection(ValidateOffset(selEndPos.Second), ValidateOffset(selStartPos.First));
+			// set the selection only if the end position (which is the start of the selection)
+			// is within the file's limits
+			if (selEndPos.Second < dataView.Buffer.Size)
+				dataView.SetSelection(ValidateOffset(selEndPos.Second), ValidateOffset(selStartPos.First));
+			else
+				dataView.SetSelection(-1, -1);
+
 			dataView.MoveCursor(off, 0);
 		}
 
 
+		//System.Console.WriteLine("Evaluate (2) Start: {0},{1},{2} End: {3},{4},{5}", selStartPos.First, selStartPos.Second, selStartPos.Digit, selEndPos.First, selEndPos.Second, selEndPos.Digit);
 	}
 
 	///<summary>Handle mouse button presses</summary>
@@ -243,11 +262,12 @@ public class DataViewControl
 				selEndPos = pos;
 			}
 		}
-
-		EvaluateSelection(DataViewDisplay.ShowType.Closest);
-
+		
 		// give the focus to the appropriate area
+		// do this before setting the new cursor/selection
 		UpdateFocus(clickArea);
+		
+		EvaluateSelection(DataViewDisplay.ShowType.Closest);
 	}
 
 	///<summary>Handle mouse motion</summary>
@@ -287,10 +307,11 @@ public class DataViewControl
 		// for better visual result
 		selEndPos = pos;
 
-		EvaluateSelection(DataViewDisplay.ShowType.Closest);
-
 		// give the focus to the appropriate area
+		// do this before setting the new cursor/selection
 		UpdateFocus(clickArea);
+		
+		EvaluateSelection(DataViewDisplay.ShowType.Closest);
 	}
 
 	///<summary>Handle mouse button release</summary>
@@ -325,53 +346,14 @@ public class DataViewControl
 		// for better visual result
 		selEndPos = pos;
 		//System.Console.WriteLine("Start: {0},{1},{2} End: {3},{4},{5}", selStartPos.First, selStartPos.Second, selStartPos.Digit, selEndPos.First, selEndPos.Second, selEndPos.Digit);
-		EvaluateSelection(DataViewDisplay.ShowType.Closest);
-
+		
 		// give the focus to the appropriate area
+		// do this before setting the new cursor/selection
 		UpdateFocus(clickArea);
+		
+		EvaluateSelection(DataViewDisplay.ShowType.Closest);
 	}
-
-	///<summary>Find the area that has the focus and its index</summary>
-	internal int FindFocusedArea(out Area focusArea)
-	{
-		int faIndex = 0;
-		focusArea = null;
-
-		for (; faIndex < dvDisplay.Layout.Areas.Count; faIndex++) {
-			if ((dvDisplay.Layout.Areas[faIndex] as Area).HasCursorFocus == true) {
-				focusArea = (dvDisplay.Layout.Areas[faIndex] as Area);
-				break;
-			}
-		}
-		if (focusArea != null)
-			return faIndex;
-		else
-			return -1;
-	}
-
-	///<summary>Give the focus to the next applicable area</summary>
-	Area CycleFocus()
-	{
-		Area focusArea = null;
-		int faIndex = FindFocusedArea(out focusArea);
-
-		int start = faIndex;
-		int areaCount = dvDisplay.Layout.Areas.Count;
-
-		for (faIndex++; (faIndex % areaCount) != start; faIndex++) {
-			Area a = (dvDisplay.Layout.Areas[faIndex%areaCount] as Area);
-			if (a.CanFocus == true) {
-				a.HasCursorFocus = true;
-				if (focusArea != null)
-					focusArea.HasCursorFocus = false;
-				dataView.MoveCursor(dataView.CursorOffset, dataView.CursorDigit);
-				return a;
-			}
-		}
-
-		return null;
-	}
-
+	
 	// variables that are shared by OnKeyPress and OnKey* handler methods
 	Area okp_focusArea;
 	int okp_bpr;
@@ -387,17 +369,19 @@ public class DataViewControl
 		bool shiftPressed = false;
 
 		// find focused area
-		FindFocusedArea(out okp_focusArea);
+		okp_focusArea = dvDisplay.Layout.AreaGroup.FocusedArea;
 
 		// if no area has the focus, give it to the first one applicable
-		if (okp_focusArea == null)
-			okp_focusArea = CycleFocus();
+		if (okp_focusArea == null) {
+			dvDisplay.Layout.AreaGroup.CycleFocus();
+			okp_focusArea = dvDisplay.Layout.AreaGroup.FocusedArea;
+		}
 
 
 		// if still no area got the focus give up
 		if (okp_focusArea == null)
 			return;
-
+		
 		okp_bpr = okp_focusArea.BytesPerRow;
 		okp_dpb = okp_focusArea.DigitsPerByte;
 		okp_showType = DataViewDisplay.ShowType.Closest;
@@ -408,7 +392,7 @@ public class DataViewControl
 		UpdateSelection(true);
 
 		Position cur = new Position();
-		if (shiftPressed || !okp_focusArea.Selection.IsEmpty()) {
+		if (shiftPressed || !dvDisplay.Layout.AreaGroup.Selection.IsEmpty()) {
 			cur = selEndPos;
 			cur.Digit = 0;
 		}
@@ -455,6 +439,11 @@ public class DataViewControl
 				break;
 			case Gdk.Key.Tab:
 				OnKeyTab();
+				// cycling through the areas may change the current cursor digit
+				// (note however that we don't want to call EvaluateSelection()->SetCursor()
+				//  because that will reset the digit of all areas. So we just update
+				//  selStartPos.Digit here and set specialKey = true to avoid calling SetCursor()...)
+				selStartPos.Digit = dvDisplay.Layout.AreaGroup.CursorDigit;
 				specialKey = true;
 				break;
 			case Gdk.Key.BackSpace:
@@ -515,7 +504,7 @@ public class DataViewControl
 	///</summary>
 	internal void OnFocusInEvent (object o, FocusInEventArgs args)
 	{
-		foreach(Area a in dvDisplay.Layout.Areas)
+		foreach(Area a in dvDisplay.Layout.AreaGroup.Areas)
 		a.IsActive = true;
 
 		dataView.FireFocusChangedEvent();
@@ -526,7 +515,7 @@ public class DataViewControl
 	///</summary>
 	internal void OnFocusOutEvent (object o, FocusOutEventArgs args)
 	{
-		foreach(Area a in dvDisplay.Layout.Areas)
+		foreach(Area a in dvDisplay.Layout.AreaGroup.Areas)
 		a.IsActive = false;
 
 		dataView.FireFocusChangedEvent();
@@ -684,7 +673,7 @@ public class DataViewControl
 
 	void OnKeyTab()
 	{
-		CycleFocus();
+		dvDisplay.Layout.AreaGroup.CycleFocus();
 	}
 
 	void OnKeyBackspace()
@@ -699,7 +688,13 @@ public class DataViewControl
 			specialKey = true;
 			return;
 		}
-
+		
+		// if the buffer isn't resizable, ignore non-overwriting keypresses
+		if (!dataView.Buffer.IsResizable && !dataView.Overwrite) {
+			specialKey = true;
+			return;
+		}
+		
 		if (dataView.Selection.IsEmpty()) {
 			if (imContext.FilterKeypress(e) && okp_focusArea.HandleKey(e.Key, dataView.Overwrite) == true) {
 				OnKeyRight(ref cur, ref next);
@@ -714,14 +709,39 @@ public class DataViewControl
 				specialKey = true;
 		}
 		else {
-			if (imContext.FilterKeypress(e) && okp_focusArea.HandleKey(e.Key, dataView.Overwrite) == true) {
-				dataView.Delete();
+			dataView.Buffer.BeginActionChaining();
+
+			// Insert the new data and delete the old
+			if (imContext.FilterKeypress(e) && okp_focusArea.HandleKey(e.Key, false) == true) {
+				Util.Range curSel = dataView.Selection;
+				long curOffset = dataView.CursorOffset;
+				
+				// move cursor to the right
+				OnKeyRight(ref cur, ref next);
+
+				next.First = next.Second = curSel.Start;
+				selEndPos = selStartPos = next;
+
+				// the new data could have been inserted either just after the end
+				// or at the beginning of the selection. Handle each case 
+				// and delete the old data.
+				if (curOffset > curSel.End) { // new data just after end
+					dataView.Delete();
+				}
+				else { // curOffset == curSel.Start, new data at the beginning
+					// shift selection one position to the right
+					dataView.SetSelection(curSel.Start + 1, curSel.End + 1);
+					dataView.Delete();
+				}
+
 				specialKey = false;
 			}
 			else {
 				specialKey = true;
 			}
 
+			dataView.Buffer.EndActionChaining();
+
 		}
 		// any other key pass it to focused area
 		// if area handled it move one position right
@@ -737,4 +757,4 @@ public class DataViewControl
 
 } // end DataViewControl
 
-} //end namespace
\ No newline at end of file
+} //end namespace
