@@ -38,10 +38,10 @@ public class FindReplacePlugin : GuiPlugin
 	DataBook dataBook;
 	FindReplaceWidget widget;
 
-	Action FindAction;
-	Action FindNextAction;
-	Action FindPreviousAction;
-	Action ReplaceAction;
+	Gtk.Action FindAction;
+	Gtk.Action FindNextAction;
+	Gtk.Action FindPreviousAction;
+	Gtk.Action ReplaceAction;
 
 	IFinder finder;
 	Window mainWindow;
@@ -124,17 +124,19 @@ public class FindReplacePlugin : GuiPlugin
 
 	void OnBufferContentsChanged(ByteBuffer bb)
 	{
-		DataView dv = null;
+		Gtk.Application.Invoke(delegate {
+			DataView dv = null;
 
-		// find DataView that owns bb
-		foreach (DataViewDisplay dvtemp in dataBook.Children) {
-			if (dvtemp.View.Buffer == bb) {
-				dv = dvtemp.View;
-				break;
+			// find DataView that owns bb
+			foreach (DataViewDisplay dvtemp in dataBook.Children) {
+				if (dvtemp.View.Buffer == bb) {
+					dv = dvtemp.View;
+					break;
+				}
 			}
-		}
 
-		UpdateActions(dv);
+			UpdateActions(dv);
+		});
 	}
 
 	void OnSwitchPage(object o, SwitchPageArgs args)
@@ -162,10 +164,10 @@ public class FindReplacePlugin : GuiPlugin
 		uim.InsertActionGroup(group, 0);
 		uim.AddUiFromString(uiXml);
 
-		FindAction = (Action)uim.GetAction("/menubar/Search/Find");
-		FindNextAction = (Action)uim.GetAction("/menubar/Search/FindNext");
-		FindPreviousAction = (Action)uim.GetAction("/menubar/Search/FindPrevious");
-		ReplaceAction = (Action)uim.GetAction("/menubar/Search/Replace");
+		FindAction = (Gtk.Action)uim.GetAction("/menubar/Search/Find");
+		FindNextAction = (Gtk.Action)uim.GetAction("/menubar/Search/FindNext");
+		FindPreviousAction = (Gtk.Action)uim.GetAction("/menubar/Search/FindPrevious");
+		ReplaceAction = (Gtk.Action)uim.GetAction("/menubar/Search/Replace");
 
 		uim.EnsureUpdate();
 
@@ -387,9 +389,7 @@ public class FindReplaceWidget : Gtk.HBox
 
 		// load selection only if it isn't very large
 		if (this.Sensitive == true && dv.Selection.Size <= 1024 && dv.Selection.Size > 0) {
-			// make sure selection is sorted
-			Bless.Util.Range sel = new Bless.Util.Range(dv.Selection);
-			sel.Sort();
+			Bless.Util.Range sel = dv.Selection;
 
 			byte[] ba = new byte[sel.Size];
 			for (int i = 0;i < sel.Size;i++) {
